import os
import cv2
from pathlib import Path
import numpy as np
import damei as dm
from copy import deepcopy

import torch.backends.cudnn as cudnn
from .datasets import LoadStreams, LoadImages


class Dataloader(object):
    def __init__(self, source=None, imgsz=None):
        self.video_suffix = ['.mp4', '.avi', '.wmv']
        self.imgsz = imgsz

        if isinstance(source, str):
            self.dataset = self.get_dataset(source)
        elif isinstance(source, list):
            datasets = [None] * len(source)
            for i, s in enumerate(source):
                datasets[i] = self.get_dataset(s)
            self.dataset = MultiDatasets(datasets)
        else:
            raise NotImplementedError(f'source "{source}" type {type(source)} not supported')

    def get_dataset(self, source):
        imgsz = self.imgsz
        video_suffix = self.video_suffix
        webcam = source.isnumeric() or source.startswith(('rtsp://', 'rtmp://', 'http://')) or source.endswith('.txt')
        if webcam:
            cudnn.benchmark = True  # set True to speed up constant image size inference
            dataset = LoadStreams(source, img_size=imgsz, flip=False)
            dataset.is_camera = True
            dataset.is_video = False
        else:
            dataset = LoadImages(source, img_size=imgsz)
            dataset.is_camera = False
            dataset.is_video = True if Path(source).suffix in video_suffix else False
        return dataset

    def __len__(self):
        return len(self.dataset)

    def __iter__(self):
        return self.dataset.__iter__()

    def __next__(self):
        return self.dataset.__next__()


class MultiDatasets(object):

    def __init__(self, datasets):
        self.datasets = datasets
        # print(f'multi {self.datasets}')
        self.is_camera = False
        self.is_video = False
        self.count = -1

    def __len__(self):
        datasets = self.datasets
        return int(np.mean([len(x) for x in datasets]))

    def __iter__(self):
        return self

    def __next__(self):
        datasets = self.datasets
        num = len(self.datasets)
        self.count += 1
        imgs = [None] * num
        im0s = [None] * num
        paths = [None] * num
        vid_caps = [None] * num
        for i, dset in enumerate(datasets):
            # 不同的数据集长度不一样呀可能会报错
            path, imgs[i], im0s[i], vid_caps[i] = next(dset)
            if dset.is_camera:
                im0s = im0s[0]
                path = f'{path[0]}/{self.count:0>6}.jpg'
            elif dset.is_video:
                path = f'{path}/{self.count:0>6}.jpg'
            else:
                pass
            paths[i] = path
        # print('\nxx', path)
        imgs = np.array(imgs)
        im0s = np.array(im0s)
        # print('xxx', imgs.shape, im0s.shape)
        return paths, imgs, im0s, vid_caps


def load_single_img(file, img_sz=640, return_list=False):
    if not os.path.exists(file):
        return
    im0 = cv2.imread(file)
    img, ratio, (dw, dh), recover = dm.general.letterbox(deepcopy(im0), new_shape=img_sz)
    img = img[:, :, ::-1].transpose(2, 0, 1)
    img = np.ascontiguousarray(img)

    if return_list:
        return [file], img[np.newaxis, :], [im0]
    else:
        return file, img, im0



# coding=utf-8

import os, sys
from pathlib import Path
import hai

pydir = Path(os.path.abspath(__file__)).parent

try:
    import damei as dm
except:
    sys.path.append(f'{Path(pydir).parent}/damei')
    import damei as dm

logger = dm.getLogger('dm.nn.test')


class TestModule(object):
    def __init__(self):
        self.uaii = hai.UAII()
        self.current_model_name = None
        self._model = None
        self.test_items = ['init', 'train', 'infer', 'evaluate']
        self.test_results = {}

        # print(self.uaii.ps())

    @property
    def model(self):
        return self._model

    def __call__(self, model_name, *args, **kwargs):
        uaii = self.uaii
        logger.info(f'Staring module: "{model_name}" testing ...')
        self.try_test(model_name, *args, **kwargs)

        print(f'test_ret: {self.test_results}')
        
    def try_test(self, model_name, *args, **kwargs):
        all_models = hai.MODULES
        print(all_models)
        self.current_model_name = model_name

        # test init
        try:
            self.test_init(*args, **kwargs)
            self.test_results['init'] = 'Passed'
        except Exception as e:
            self.test_results['init'] = f'Failed with error: {e}'
            raise e

        # test train
        try:
            self.test_train(*args, **kwargs)
            self.test_results['train'] = 'Passed'
        except Exception as e:
            self.test_results['train'] = f'Failed with error: {e}'
            raise e


    def test_train(self):
        model = self.model
        model.train()

    def test_init(self):
        # test model load
        model = self.uaii.load_model(self.current_model_name)
        print('model', model)
        # test config
        config = model.config
        print('config', config)
        self._model = model

    def test_module(self, m_name, **kwargs):
        uaii = self.uaii
        logger.info(f'测试模块{m_name}初始化。')
        # module = uaii.get_module(name=m_name)()  # 获取并初始化
        stream = uaii.build_stream(m_name)
        return stream


    def x(self):
        cdir = os.getcwd()
        # 检查是否有dmapi
        dirs = [x for x in os.listdir('.') if os.path.isdir(x)]
        if 'dmapi' not in dirs:
            raise NotADirectoryError(
                f'Please run this tool on the directory which contains the "dmapi" fold. '
                f'Your can refer to the damei doc for configuration method.',
            )
        if cdir not in sys.path:  # 添加当前路径，以便能from dmapi import *
            sys.path.append(f'{cdir}')


if __name__ == '__main__':
    tm = TestModule()
    tm()



import os, sys
from pathlib import Path
from hai.version import __appname__

ROOT_PATH = f'{Path(os.path.abspath(__file__)).parent.parent.parent.parent}'  # /home/xxx/VSProjects/hai
WEIGHTS_ROOT = f'{os.environ["HOME"]}/.{__appname__}/weights'  # /home/xxx/.hai/weights
DATASETS_ROOT = f'{os.environ["HOME"]}/datasets/{__appname__}_datasets'  # /home/xxx/.hai/datasets
API_FOLD_NAME = f'{__appname__}_api'  # hai_api

